@@ -26,9 +26,9 @@
 #include "axel.h"
 
 static void stop( int signal );
-static char *size_human( int value );
+static char *size_human( long long int value );
 static char *time_human( int value );
-static void print_commas( int bytes_done );
+static void print_commas( long long int bytes_done );
 static void print_alternate_output( axel_t *axel );
 static void print_help();
 static void print_version();
@@ -52,6 +52,8 @@ static struct option axel_options[] =
 	{ "help",		0,	NULL,	'h' },
 	{ "version",		0,	NULL,	'V' },
 	{ "alternate",		0,	NULL,	'a' },
+	{ "header",		1,	NULL,	'H' },
+	{ "user-agent",		1,	NULL,	'U' },
 	{ NULL,			0,	NULL,	0 }
 };
 #endif
@@ -67,7 +69,7 @@ int main( int argc, char *argv[] )
 	search_t *search;
 	conf_t conf[1];
 	axel_t *axel;
-	int i, j;
+	int i, j, cur_head = 0;
 	char *s;
 	
 #ifdef I18N
@@ -88,12 +90,18 @@ int main( int argc, char *argv[] )
 	{
 		int option;
 		
-		option = getopt_long( argc, argv, "s:n:o:S::NqvhHVa", axel_options, NULL );
+		option = getopt_long( argc, argv, "s:n:o:S::NqvhVaH:U:", axel_options, NULL );
 		if( option == -1 )
 			break;
 		
 		switch( option )
 		{
+		case 'U':
+			strncpy( conf->user_agent, optarg, MAX_STRING);
+			break;
+		case 'H':
+			strncpy( conf->add_header[cur_head++], optarg, MAX_STRING );
+			break;
 		case 's':
 			if( !sscanf( optarg, "%i", &conf->max_speed ) )
 			{
@@ -152,7 +160,7 @@ int main( int argc, char *argv[] )
 			return( 1 );
 		}
 	}
-	
+	conf->add_header_count = cur_head;
 	if( j > -1 )
 		conf->verbose = j;
 	
@@ -164,11 +172,19 @@ int main( int argc, char *argv[] )
 	else if( strcmp( argv[optind], "-" ) == 0 )
 	{
 		s = malloc( MAX_STRING );
-		scanf( "%127[^\n]s", s );
+		if (scanf( "%1024[^\n]s", s) != 1) {
+			fprintf( stderr, _("Error when trying to read URL (Too long?).\n") );
+			return( 1 );
+		}
 	}
 	else
 	{
 		s = argv[optind];
+		if( strlen( s ) > MAX_STRING )
+		{
+			fprintf( stderr, _("Can't handle URLs of length over %d\n" ), MAX_STRING );
+			return( 1 );
+		}
 	}
 	
 	printf( _("Initializing download: %s\n"), s );
@@ -246,8 +262,17 @@ int main( int argc, char *argv[] )
 		{
 			if( S_ISDIR( buf.st_mode ) )
 			{
-				strncat( fn, "/", MAX_STRING );
-				strncat( fn, axel->filename, MAX_STRING );
+				size_t fnlen = strlen(fn);
+				size_t axelfnlen = strlen(axel->filename);
+				
+				if (fnlen + 1 + axelfnlen + 1 > MAX_STRING) {
+					fprintf( stderr, _("Filename too long!\n"));
+					return ( 1 );
+				}
+				
+				fn[fnlen] = '/';
+				memcpy(fn+fnlen+1, axel->filename, axelfnlen);
+				fn[fnlen + 1 + axelfnlen] = '\0';
 			}
 		}
 		sprintf( string, "%s.st", fn );
@@ -318,7 +343,7 @@ int main( int argc, char *argv[] )
 	
 	while( !axel->ready && run )
 	{
-		int prev, done;
+		long long int prev, done;
 		
 		prev = axel->bytes_done;
 		axel_do( axel );
@@ -342,9 +367,9 @@ int main( int argc, char *argv[] )
 						if( prev >= 1024 )
 							printf( "  [%6.1fKB/s]", (double) axel->bytes_per_second / 1024 );
 						if( axel->size < 10240000 )
-							printf( "\n[%3i%%]  ", min( 100, 102400 * i / axel->size ) );
+							printf( "\n[%3lld%%]  ", min( 100, 102400 * i / axel->size ) );
 						else
-							printf( "\n[%3i%%]  ", min( 100, i / ( axel->size / 102400 ) ) );
+							printf( "\n[%3lld%%]  ", min( 100, i / ( axel->size / 102400 ) ) );
 					}
 					else if( ( i % 10 ) == 0 )
 					{
@@ -408,12 +433,12 @@ void stop( int signal )
 }
 
 /* Convert a number of bytes to a human-readable form			*/
-char *size_human( int value )
+char *size_human( long long int value )
 {
 	if( value == 1 )
-		sprintf( string, _("%i byte"), value );
+		sprintf( string, _("%lld byte"), value );
 	else if( value < 1024 )
-		sprintf( string, _("%i bytes"), value );
+		sprintf( string, _("%lld bytes"), value );
 	else if( value < 10485760 )
 		sprintf( string, _("%.1f kilobytes"), (float) value / 1024 );
 	else
@@ -439,7 +464,7 @@ char *time_human( int value )
 
 /* Part of the infamous wget-like interface. Just put it in a function
 	because I need it quite often..					*/
-void print_commas( int bytes_done )
+void print_commas( long long int bytes_done )
 {
 	int i, j;
 	
@@ -457,8 +482,8 @@ void print_commas( int bytes_done )
 
 static void print_alternate_output(axel_t *axel) 
 {
-	int done=axel->bytes_done;
-	int total=axel->size;
+	long long int done=axel->bytes_done;
+	long long int total=axel->size;
 	int i,j=0;
 	double now = gettime();
 	
@@ -484,7 +509,12 @@ static void print_alternate_output(axel_t *axel)
 			putchar(' ');
 	}
 	
-	printf( "] [%6.1fKB/s]", (double) axel->bytes_per_second / 1024 );
+	if(axel->bytes_per_second > 1048576)
+		printf( "] [%6.1fMB/s]", (double) axel->bytes_per_second / (1024*1024) );
+	else if(axel->bytes_per_second > 1024)
+		printf( "] [%6.1fKB/s]", (double) axel->bytes_per_second / 1024 );
+	else
+		printf( "] [%6.1fB/s]", (double) axel->bytes_per_second );
 	
 	if(done<total)
 	{
@@ -513,6 +543,8 @@ void print_help()
 		"-n x\tSpecify maximum number of connections\n"
 		"-o f\tSpecify local output file\n"
 		"-S [x]\tSearch for mirrors and download from x servers\n"
+		"-H x\tAdd header string\n"
+		"-U x\tSet user agent\n"
 		"-N\tJust don't use any proxy server\n"
 		"-q\tLeave stdout alone\n"
 		"-v\tMore status information\n"
@@ -520,7 +552,7 @@ void print_help()
 		"-h\tThis information\n"
 		"-V\tVersion information\n"
 		"\n"
-		"Report bugs to lintux@lintux.cx\n") );
+		"Feel free to report bugs - <pedramveisi@gmail.com>\n") );
 #else
 	printf(	_("Usage: axel [options] url1 [url2] [url...]\n"
 		"\n"
@@ -528,6 +560,8 @@ void print_help()
 		"--num-connections=x\t-n x\tSpecify maximum number of connections\n"
 		"--output=f\t\t-o f\tSpecify local output file\n"
 		"--search[=x]\t\t-S [x]\tSearch for mirrors and download from x servers\n"
+		"--header=x\t\t-H x\tAdd header string\n"
+		"--user-agent=x\t\t-U x\tSet user agent\n"
 		"--no-proxy\t\t-N\tJust don't use any proxy server\n"
 		"--quiet\t\t\t-q\tLeave stdout alone\n"
 		"--verbose\t\t-v\tMore status information\n"
@@ -535,14 +569,13 @@ void print_help()
 		"--help\t\t\t-h\tThis information\n"
 		"--version\t\t-V\tVersion information\n"
 		"\n"
-		"Report bugs to lintux@lintux.cx\n") );
+		"Feel free to report bugs - <pedramveisi@gmail.com>\n") );
 #endif
 }
 
 void print_version()
 {
 	printf( _("Axel version %s (%s)\n"), AXEL_VERSION_STRING, ARCH );
-	printf( "\nCopyright 2001-2002 Wilmer van der Gaast.\n" );
 }
 
 /* Print any message in the axel structure				*/
